/*
 *  CPAchecker is a tool for configurable software verification.
 *  This file is part of CPAchecker.
 *
 *  Copyright (C) 2007-2018  Dirk Beyer
 *  All rights reserved.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *
 *  CPAchecker web page:
 *    http://cpachecker.sosy-lab.org
 */
 
parser grammar LtlGrammarParser;

options {
  tokenVocab = LtlLexer;
  language = Java;
}

/*
 * Parser Rules
 */

property : CHECK LPAREN initFunction COMMA ltlProperty RPAREN EOF ;

initFunction : INIT LPAREN FUNCTIONNAME RPAREN ;

ltlProperty : LTL LPAREN root=expression RPAREN ;

formula : root=expression EOF ;

expression : orExpression ;

orExpression : andExpression (OR andExpression)* ;

andExpression : binaryExpression (AND binaryExpression)* ;

binaryExpression : left=unaryExpression op=binaryOp right=binaryExpression # binaryOperation
                 | unaryExpression # binaryUnary
                 ;

unaryExpression : op=unaryOp inner=binaryExpression # unaryOperation
                | atomExpression # unaryAtom
                ;

atomExpression : constant=bool # boolean
               | QUOTATIONMARK_START var=PARAM comp=COMPARATOR val=VALUE (MATHOP VALUE)* QUOTATIONMARK_END # quotedVariable
               | var=VARIABLE # variable
               | LPAREN nested=expression RPAREN # nested
               ;

unaryOp : NOT
        | FINALLY
        | GLOBALLY
        | NEXT
        ;

binaryOp : EQUIV
         | IMP
         | XOR
         | UNTIL
         | WUNTIL
         | RELEASE
         | SRELEASE
         ;

bool : TRUE
     | FALSE
     ;

